/* This file is part of Threadloger.
 *
 * Threadloger is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Threadloger is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Threadloger; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
#include <stdlib.h>
#include <stdarg.h>
#include <assert.h>
#include <stdio.h>		// We log onto stderr
#include <string.h>
#include <limits.h>
#include <errno.h>
#include <unistd.h>
#include <dirent.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "threadlog/logd.h"

/*
 * Data Definitions
 */

static char const Name[] = "$Name$", Id[] = "$Id$";

struct log_ops log_ops = {
	.print = NULL,		// pure virtual
};

/*
 * Private Functions
 */

/*
 * Public Functions
 */

int log_ctor(struct log *log)
{
	assert(log);
	log->ops = &log_ops;
	log->level = log_WARNING;
	return 0;
}

void log_dtor(struct log *log)
{
	(void)log;
}

extern inline void log_set_level(struct log *log, enum log_level level);

extern inline enum log_level log_get_level(struct log const *log);

char const *log_level_str(enum log_level level)
{
	switch (level) {
		case log_ALWAYS:
			return "ALW";
		case log_ERROR:
			return "ERR";
		case log_WARNING:
			return "WRN";
		case log_NOTICE:
			return "NOT";
		case log_DEBUG:
			return "DBG";
	}
	assert(0);
	return "BAD";
}

extern inline bool logid_reserved(logid_t id);

