/* This file is part of Threadloger.
 *
 * Threadloger is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Threadloger is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Threadloger; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
/** @file logc.h
 * @brief Log client
 * @version $Id: logc.h,v 1.1.1.1 2006/09/06 16:46:49 ccellier Exp $
 *
 * Use this implementation of log to send your log
 * message to loger.
 *
 * To log a message, use logc.log.ops->print.
 */
#ifndef LOGC_H_060825
#define LOGC_H_060825

#include <netinet/in.h>
#include <threadlog/log.h>

#define LOG_LOCAL_ID_MAX 8	///< Max size of a module's short name.

struct logc;

/** Constructor.
 */
int logc_ctor(
	struct logc *logc,	///< The logc structure to initialize.
	char (*local_id)[LOG_LOCAL_ID_MAX],	///< A short name that identify your module.
	char const *server_name,	///< Hostname of loger.
	unsigned short port	///< Port used by loger.
);

/** Destructor.
 */
void logc_dtor(
	struct logc *logc	///< The logc structure to destroy.
);

/** Used as a message header in the datagram.
 */
struct dtg_hdr {
	enum log_level level;	///< Log level of this message.
	time_t date;		///< Timestamp of this message (local to sender).
	logid_t this_logid;		///< Local id given to this message by logc implementation.
	logid_t refs[LOG_REFS_MAX];	///< References for this message (local ids).
	char local_id[LOG_LOCAL_ID_MAX];	///< Short name of the sender.
	ssize_t str_size;		///< Size of the folowing string, including NUL trailing char.
	char str[];			///< Message.
};

/** Used internally as a datagram body.
 */
struct dtg_max {
	struct dtg_hdr hdr;		///< Header of the message.
	char string[LOG_STRING_MAX];	///< String of the message. The actual datagrams may be shorter.
};

/** This object is supposed to be opaque.
 */
struct logc {
	struct log log;		///< the log object.
	int sockfd;			///< The socket toward loger daemon.
	struct sockaddr_in server;	///< Address of the server, just in case.
	logid_t seq;		///< A sequence used to create local message ids.
	char (*local_id)[LOG_LOCAL_ID_MAX];	///< Used by server to identify logers from same host. If severall instance of the same prog use logc, they must change their local_id.
};

#endif
