/* This file is part of Threadloger.
 *
 * Threadloger is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Threadloger is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Threadloger; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
/** @file logd.h
 * @brief Implementation of log that write in local files.
 * @version $Id: logd.h,v 1.1.1.1 2006/09/06 16:46:49 ccellier Exp $
 *
 * Logd print function writes log messages on various local files,
 * all located in a single directory.
 * This is not safe on multiprogramming (nor multithread). To have
 * severall programs log in the same directory, use logc and the
 * loger daemon.
 */

#ifndef LOGD_H_060825
#define LOGD_H_060825

#include <stdbool.h>
#include <threadlog/log.h>

struct logd;

/** Constructor.
 */
int logd_ctor(
	struct logd *logd,	///< Logd structure to initialize.
	char const *log_path,	///< Path of the log directory.
	size_t max_filesize	///< Msg files are rotated when this size is exceeded. Given in Mb here.
);

/** Destructor.
 */
void logd_dtor(
	struct logd *log	///< Log structure to release.
);

/** You should not change anything in there.
 * To log a message, use logd.log.ops->print()
 */
struct logd {
	struct log log;		///< The log object.
	char const *path;		///< Path to the directory where log files are written.
	size_t max_filesize;	///< Above this filesize, new message files are created.
	logid_t first_id;		///< First id of the current message file.
	int msgs_fd;		///< File descriptor of the actual message file.
	logid_t default_refs[2];	///< Default references to use for messages writhout refs.
};

#endif
