/* This file is part of Threadloger.
 *
 * Threadloger is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Threadloger is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Threadloger; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
#ifndef LOGDLIB_H_060830
#define LOGDLIB_H_060830

#include <dirent.h>
#include <threadlog/logd.h>

/*
 * Data Definitions
 */

struct log_record {
	time_t date;		// must be first, or change refs_compare and get_first_date
	enum log_level level;
	logid_t refs[LOG_REFS_MAX];
	ssize_t str_size;
	char string[];
};

struct log_msg {
	logid_t id;
	unsigned nb_bl;
	struct log_record record;
	char string[LOG_STRING_MAX];
};

struct bl_curs {
	logid_t id;
	int fd;
};

struct chrono_curs {
	int fd;
	logid_t first;
};

struct msg {
	struct log_record record;
	char string[LOG_STRING_MAX];
};

/*
 * Prototypes
 */

int log_msg_ctor(struct log_msg *msg, logid_t id, char const *path);

void log_msg_dtor(struct log_msg *msg);

int bl_curs_ctor(struct bl_curs *curs, logid_t id, char const *path);

void bl_curs_dtor(struct bl_curs *curs);

logid_t bl_curs_next(struct bl_curs *curs);

int chrono_curs_ctor(struct chrono_curs *curs, logid_t id, char const *path);

void chrono_curs_dtor(struct chrono_curs *curs);

logid_t chrono_curs_next(struct chrono_curs *curs);

unsigned nb_bl(logid_t id, char const *path);

int close_backlink(logid_t id, int fd);

int open_backlink(logid_t id, char const *path);

int get_backlink_name(char *str, char const *path, size_t max_size, logid_t id);

int read_msg(struct log_record *record, logid_t id, char const *path);

int get_fd_from_first_id(logid_t id, char const *path);

logid_t path_first_id(char const *path);

logid_t get_first_id(logid_t id, char const *path);

int get_last_fileid(logid_t * id, char const *path);

int get_namelist(struct dirent ***namelist, char const *path);

int read_all_filename(void *buf, char const *filename, size_t size);

int trunc_to_min_size(int fd, size_t min_size);

ssize_t filename_size(char const *file);

ssize_t file_size(int fd);

int read_all(int fd, void *buf, size_t size);

int read_all_or_EOF(int fd, void *buf, size_t size);

int write_all(int fd, void const *buf, size_t size);

int mkdir_all(char const *path);

enum log_level log_prio2level(int prio);

#endif
