/* This file is part of Threadloger.
 *
 * Threadloger is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Threadloger is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Threadloger; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <stddef.h>
#include <assert.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include "threadlog/logd.h"
#include "threadlogd.h"
#include "client.h"

/*
 * Public Functions
 */

int client_ctor(struct client *client, struct sockaddr_in const *address, char local_id[LOG_LOCAL_ID_MAX])
{
	client->address = *address;
	strncpy(client->local_id, local_id, sizeof(client->local_id));
	TAILQ_INIT(&client->seq2id_list);
	TAILQ_INIT(&client->free_list);
	for (unsigned i=sizeof_array(client->seq2ids); i--; ) {
		TAILQ_INSERT_HEAD(&client->free_list, client->seq2ids+i, item);
	}
	return 0;
}

void client_dtor(struct client *client)
{
	(void)client;
}

void client_reset(struct client *client, char local_id[LOG_LOCAL_ID_MAX])
{
	struct sockaddr_in addr = client->address;
	client_dtor(client);
	(void)client_ctor(client, &addr, local_id);
}

struct seq2id *client_find_seq2id(struct client *client, logid_t seq)
{
	assert(client);
	struct seq2id *seq2id;
	TAILQ_FOREACH(seq2id, &client->seq2id_list, item) {
		if (seq2id->seq == seq) {
			return seq2id;
		}
	}
	return NULL;
}

void client_promote_seq2id(struct client *client, struct seq2id *seq2id)
{
	assert(client && seq2id);
	TAILQ_REMOVE(&client->seq2id_list, seq2id, item);
	TAILQ_INSERT_HEAD(&client->seq2id_list, seq2id, item);
}

void client_add_seq2id(struct client *client, logid_t seq, logid_t id)
{
	assert(client);
	struct seq2id *seq2id;
	if (TAILQ_EMPTY(&client->free_list)) {
		// replace oldest
		seq2id = TAILQ_LAST(&client->seq2id_list, seq2id_list);
		client_promote_seq2id(client, seq2id);
	} else {
		seq2id = TAILQ_FIRST(&client->free_list);
		TAILQ_REMOVE(&client->free_list, seq2id, item);
		TAILQ_INSERT_HEAD(&client->seq2id_list, seq2id, item);
	}
	seq2id->seq = seq;
	seq2id->id = id;
}

char const *client_name(struct client *client)
{
	assert(client);
	static char str[32];
	snprintf(str, sizeof(str), "%.*s(%s)",
		sizeof(client->local_id),
		client->local_id,
		inet_ntoa(client->address.sin_addr));
	return str;
}
