/* This file is part of Threadloger.
 *
 * Threadloger is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Threadloger is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Threadloger; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
#ifndef CLIENT_H_060825
#define CLIENT_H_060825

#include <netinet/in.h>
#include "threadlog/queue.h"
#include "threadlog/logc.h"

struct client {
	logid_t connect_id;
	struct sockaddr_in address;	// client's address
	char local_id[LOG_LOCAL_ID_MAX];
	TAILQ_HEAD(seq2id_list, seq2id) seq2id_list;	// list of used seq2id structuces (more recently used first).
	TAILQ_HEAD(free_list, seq2id) free_list;	// list of unused seq2id structuces
	struct seq2id {
		TAILQ_ENTRY(seq2id) item;
		logid_t seq;		// set by the client
		logid_t id;		// actual logid
	} seq2ids[1024];
};

int client_ctor(struct client *client, struct sockaddr_in const *address, char local_id[LOG_LOCAL_ID_MAX]);

void client_dtor(struct client *client);

void client_reset(struct client *client, char local_id[LOG_LOCAL_ID_MAX]);

struct seq2id *client_find_seq2id(struct client *client, logid_t seq);

void client_promote_seq2id(struct client *client, struct seq2id *seq2id);

void client_add_seq2id(struct client *client, logid_t seq, logid_t logid);

// returns a static buffer
char const *client_name(struct client *client);

#endif
